
class TagNode extends Node {
  TagGraph graph;

  PImage picture, content;
  String img, img2;
  GLTexture srcTex;

  GLGraphicsOffScreen pg;

  float maxZoom=1;
  float minZoom=0.5;
  float pictureZoom = minZoom;
  float xCord = 0;
  float yCord = 0;

  float reverseSpeed = -.4;
  float slowSpeed = 0.985;
  float bounce = 0;

  float rotation = random(-90, 90);
  boolean doRotate = false;
  float zDepth = 0;

  float phispeed = 0;
  float gravity = .7;
  float friction = .9999;
  float ymin = 360;
  float yspeed = 0;
  float ymax = 0;
  int shapeCount = 0;

  float rotationTemp = rotation;
  float easing = 0.005;

  int tempT = 0;
  boolean doFading = false;
  boolean fadeFirst = true;

  float tempZDistance = 0;


  int picHeight = picSize;
  int picWidth = picSize;


  boolean coordsSet = false;

  TagNode(TagGraph theGraph) {
    super();
    graph = theGraph;
    init();
  }

  TagNode(TagGraph theGraph, float theX, float theY) {
    super(theX, theY);
    graph = theGraph;
    init();
  }

  TagNode(TagGraph theGraph, float theX, float theY, float theZ) {
    super(theX, theY, theZ);
    graph = theGraph;
    init();
  }

  TagNode(TagGraph theGraph, PVector theVector) {
    super(theVector);
    graph = theGraph;
    init();
  }

  void setID(String theID) {
    super.setID(theID);
  }

  void init() {

    picHeight = picWidth = int(random(picSize, picSize+51));
  }

  void draw() {


    if (picture != null)
    {
      canvas.beginDraw();
      canvas.pushMatrix();
      canvas.translate(x, y, map(pictureZoom, minZoom, maxZoom, zLevel, 0));
      canvas.noStroke();
      canvas.rotateZ(radians(map(pictureZoom, minZoom, maxZoom, rotation, 0)));
      canvas.scale(pictureZoom);
      canvas.beginShape();
      int aValue = int(map(pictureZoom, minZoom, 0.9*maxZoom, 160, 255));
      int aValue2 = int(map(pictureZoom, 0.9*maxZoom, maxZoom, 0, 255));

      if (id == "iad_title")
      {
        canvas.fill(150);
      }
      else {
        canvas.fill(100, aValue);
      }
      canvas.vertex(-picWidth/2-10, -picHeight/2-10, 0);
      canvas.vertex(picWidth/2+10, -picHeight/2-10, 0);
      canvas.vertex(picWidth/2+10, picHeight/2+10, 0);
      canvas.vertex(-picWidth/2-10, picHeight/2+10, 0);
      canvas.endShape();

      canvas.beginShape();
      canvas.tint(255, aValue);
      canvas.texture(picture);
      canvas.vertex(-picWidth/2, -picHeight/2, 0, 0, 0);
      canvas.vertex(picWidth/2, -picHeight/2, 0, picture.width, 0);
      canvas.vertex(picWidth/2, picHeight/2, 0, picture.width, picture.height);
      canvas.vertex(-picWidth/2, picHeight/2, 0, 0, picture.height);
      canvas.endShape();
      canvas.noTint();


      if ( pictureZoom > 0.9*maxZoom) { 


        canvas.beginShape();
        canvas.tint(255, aValue2);
        canvas.texture(content);
        canvas.vertex(-picWidth/2, -picHeight/2, 0, 0, 0);
        canvas.vertex(picWidth/2, -picHeight/2, 0, picture.width, 0);
        canvas.vertex(picWidth/2, picHeight/2, 0, picture.width, picture.height);
        canvas.vertex(-picWidth/2, picHeight/2, 0, 0, picture.height);
        canvas.endShape();
        canvas.noTint();
      }
      canvas.popMatrix();
      canvas.endDraw();
    }
    else
    {
    }
  }


  void drawLabel() {
    // draw text

    textAlign(LEFT);
    rectMode(CORNER);
    float tfactor = 1;
    fill(255, 80);
    float ts = 12;
    textFont(graph.font, ts);
    //println(id);
    float tw = textWidth(id);
    rect(x+ (diameter/2+4)*tfactor, y-(ts/2)*tfactor, (tw+3)*tfactor, (ts+3)*tfactor);
    fill(80);
    text(id, x+(diameter/2+5)*tfactor, y+6*tfactor);
    // draw text for rolloverNode
  }

  void update() {
    super.update();
  }

  void setPicture(String img) {
    picture = loadImage(img);
    // srcTex = new GLTexture(root, img);
  }


  void setContent(String img2) {
    content = loadImage(img2);
    // srcTex = new GLTexture(root, img2);
  }



  void setZoomFactor(float _distance)
  {
    {
      pictureZoom = _distance;
    }
  }


  float getZoomFactor() {

    return pictureZoom;
  }

  void changeCoordsSet()
  {
    coordsSet = !coordsSet;
  }


  void changeRotation()
  {
    doRotate = !doRotate;
  }

  boolean rotationSet()
  {
    return doRotate;
  }


  boolean coordsSet()
  {
    return coordsSet;
  }

  void setCoords(float xCoord, float yCoord)
  {
    xCord = xCoord;
    yCord = yCoord;
    //println(id);
  }

  float xCoordinate()
  {
    return xCord;
  }

  float yCoordinate()
  {
    return yCord;
  }

  void zDepth(float zdepth)
  {
    zDepth = zdepth;
  }

  PImage imageBlur(float blurSize)
  {
    GLTexture destTex, tex0;
    GLTextureFilter vertBlur, horBlur, pixelate;
    vertBlur = new GLTextureFilter(root, "vertBlur.xml");
    horBlur = new GLTextureFilter(root, "horBlur.xml");
    pixelate = new GLTextureFilter(root, "pixelate.xml");


    int w = srcTex.width;
    int h = srcTex.height;
    destTex = new GLTexture(root, w, h);
    tex0 = new GLTexture(root, w, h, GLTexture.FLOAT);
    horBlur.setParameterValue("pixel_size", blurSize);
    vertBlur.setParameterValue("pixel_size", blurSize);
    srcTex.filter(horBlur, tex0);
    srcTex.filter(vertBlur, tex0);
    pixelate.setParameterValue("pixel_size", blurSize);
    //srcTex.filter(pixelate, tex0);

    destTex = tex0;


    return destTex;
  }

  void setRadius(int rradius) {

    super.setRadius(rradius);
  }

  void setStrength(int strength) {

    super.setStrength(strength);
  }

  void attract(Node nodes) {
    super.attract(nodes);
  }

  void setMaxX(int maxX) {

    super.setMaxX(maxX);
  }
  void setMaxY(int maxY) {

    super.setMaxY(maxY);
  }
  void setMinX(int minX) {

    super.setMinX(minX);
  }
  void setMinY(int minY) {

    super.setMinY(minY);
  }

  boolean isFading()
  {
    return doFading;
  }


  void changeFading()
  {  
    doFading = !doFading;
    fadeFirst = true;
    tempZDistance = z;
  }


  float fading()
  {

    if (fadeFirst)
    { 
      tempT = millis(); 
      fadeFirst = false;
    }


    return millis()-tempT;
  }

  float tempZDistance()
  {
    return tempZDistance;
  }
}

